/* For more see,
 *  https://www.arduino.cc/en/Reference/SD
 */

#include <SD.h>

//SPI Settings
#define SD_CS  10
// #define SD_DI  11
// #define SD_DO  12
// #define SD_CLK 13
// SD card gets pow and gnd from LCD shield

File root;

const int progLen = 27;
const int nBits = 16;
const PROGMEM char program[progLen][nBits] = {
  "0000000001100100",
  "1110110000010000",
  "0000000000010000",
  "1110001100001000",
  "0000000000001010",
  "1110110000010000",
  "0000000000010001",
  "1110001100001000",
  "0000000000010010",
  "1110101010001000",
  "0000000000010010",
  "1111110000010000",
  "0000000000010001",
  "1111010011010000",
  "0000000000011001",
  "1110001100000010",
  "0000000000010000",
  "1111110000010000",
  "0000000000010010",
  "1111000010100000",
  "1110111010001000",
  "0000000000010010",
  "1111110111001000",
  "0000000000001010",
  "1110101010000111",
  "0000000000011001",
  "1110101010000111",
};

void printDirectory( File dir, int numTabs ) {
  
  // https://www.arduino.cc/en/Tutorial/listfiles

  // List directory
  Serial.print( "\n" );
  for( uint8_t i = 0; i < numTabs; i++ ) {
    Serial.print( "\t" );
  }
  Serial.print( "Contents of ");
  Serial.print( dir.name() );
  Serial.println( " : \n" );

  // List entries
  dir.rewindDirectory(); // go to first file in directory
  
  while( true ) {

    File entry = dir.openNextFile();

    // no more files
    if( ! entry ) { 

      Serial.println( "" );
      break; 
    }

    for( uint8_t i = 0; i < numTabs + 1; i++ ) {
      Serial.print( "\t" );
    }
    Serial.print( entry.name() );

    if( entry.isDirectory() ) {

      // is directory
      Serial.println( "/" );
      printDirectory( entry, numTabs + 1 );
    }
    else {

      // is file
      Serial.print( "\t\t" );
      Serial.println( entry.size(), DEC );
    }

    entry.close();
  }
}

void printCardInfo(){

  // printVolume();

  root = SD.open( "/" );
  // rootsize = root.size();

  // printVolume();
  
  printDirectory( root, 0 );
  
  root.close();
}

void readLinesFromFile( String filepath ) {

  File leFile = SD.open( filepath );

  char delimeter = '\n';
  boolean reachedDelimeter = false;
  String dataString;
  char curCharacter;

  const int bufLen = 10;
  char buf[ bufLen ]; // can use char array to read chunks

  if( leFile ) {

    Serial.println( "Reading file" );

    while( leFile.available() ) { // while the file has bytes available,

      curCharacter = leFile.read();
      
      // curCharacter = leFile.peek();
      // leFile.read( buf, bufLen );

      if( curCharacter == delimeter ) {
        
        Serial.println( "Reached line end" );
        
        // stuff
        Serial.println( dataString );

        //reset
        dataString = "";        
      }
      else {

        dataString += curCharacter;
        Serial.println( dataString );

        // Serial.println( buf );
      }
    }

    leFile.close();
    
  }
  else {

    Serial.println( "Couldn't access file" );
    return NULL;    
  }
  
}

void createBinFile( String filepath ) {

  // Delete existing
  if( SD.exists( filepath ) ) SD.remove( filepath );

  // Create new
  File binFile = SD.open( filepath, FILE_WRITE );

  unsigned long pos;
  unsigned long lastPos = ( nBits + 1 ) * progLen;

  int idx = 0;

  if( binFile ) {

    Serial.print( "Started writing to ");
    Serial.println( filepath );

    char line[ nBits ];

    idx = 0;
    pos = 0;

    while ( pos < lastPos ) {

      // Get data
      for ( int j = 0; j < nBits; j++ ) {

        line[j] = pgm_read_byte_near( program[idx] + j );
      }

      // Write data
      binFile.write( line, nBits );
      binFile.write( '\n' );
      // binFile.write( '\r\n' );  // Use this instead if want Windows style line endings

      Serial.println( (String)line );

      // Update position
      pos += nBits + 1;
      binFile.seek( pos );

      idx += 1;

    }

    binFile.close();
    
    Serial.println( "Done writing to " );
    Serial.println( filepath );

  }
  else {

    Serial.println( "Couldn't access file" );
  }

  return;

}

void readBinFile( String filepath, unsigned long lineIdx ) {
  
  File binFile = SD.open( filepath, FILE_READ );

  if( binFile ) {

    Serial.print( "Started reading of ");
    Serial.println( filepath );

    unsigned long pos = lineIdx * ( nBits + 1 );
    binFile.seek( pos );

    char line[ nBits ];
    binFile.read( line, nBits );

    Serial.println( (String)line );

    binFile.close();
    
    Serial.print( "Done reading of " );
    Serial.println( filepath );

  }
  else {

    Serial.print( "Couldn't access " );
    Serial.println( filepath );
  }
}

void debugFileWrite( String filepath ) {

  File theFile = SD.open( filepath, FILE_READ );

  if( theFile ) {

    Serial.print( "Started debug of ");
    Serial.println( filepath );

    char thing;

    while ( theFile.available() ) {

      thing = theFile.read();
      Serial.print( (String)thing );
    }

    Serial.println("");

    theFile.close();
    
    Serial.print( "Done debug of " );
    Serial.println( filepath );

  }
  else {

    Serial.print( "Couldn't access " );
    Serial.println( filepath );
  }

  return;

}

void setup() {
  
  Serial.begin( 9600 );
  Serial.println( "SD test!" );

  if( !SD.begin( SD_CS ) ) {
    Serial.println( "Card not recognized" );
    return; // end program
  }

  Serial.println( "Card ready" );

  // readLinesFromFile( "quotes.txt" );

  // printCardInfo();

  // createBinFile( "crayon.txt" );

  // debugFileWrite( "crayon.txt" );

  // printCardInfo();

  // readBinFile( "crayon.txt", 0 );
  for( int i = 0; i < 28; i++ ) {
    
    readBinFile( "crayon.txt", i );
  }

}

void loop() {

/*
  String dataString = "Cats galore!";

  dataFile = SD.open( "candy.txt", FILE_WRITE ); //filenames limited to 8 chars

  if( dataFile ) {
    dataFile.println( dataString );
    dataFile.close();
    
    Serial.println(dataString);
  }
  else {
    Serial.println( "Couldn't access file" );
  }

  if( dl_time ) {
    delay( dl_time );
  }
  else {
    delay(50); //ms
  }
*/

}
